import hashlib
import logging
import os
import time
from typing import Optional

import fire
from torrentp import TorrentDownloader

from .persistentdict import PersistentDict
from .sources import MiKanProject

data = PersistentDict(os.path.expanduser("~//.bgmiget"))
logging.basicConfig(filename='bgmiget.log', level=logging.DEBUG)


class BgmiGet:

    '''
    Used to build the main command-line program.
    '''

    def __init__(self, source) -> None:
        self.source = source()

    def search(self, query: str, episode: str = "", subtitleType: str = "", subtitleGroup: str = "") -> None:
        '''
        Search anime through various meta information.
        '''
        episode = str(episode)

        timestamp = int(time.time())
        hashString = "|".join([query, episode, subtitleType, subtitleGroup])
        searchHash = hashlib.sha256(hashString.encode('utf-8')).hexdigest()

        for key in list(data.keys()):
            if isinstance(data[key], dict) and "timestamp" in data[key]:
                    if timestamp - data[key]["timestamp"] > 300:
                        del data[key]


        if searchHash in data:
            logging.info(
                f"Found {query} in cache. Timestamp: {data[searchHash]['timestamp']}")
            results = data[searchHash]["results"]
        else:
            results = self.source.search(query)
            if episode or subtitleType or subtitleGroup:
                results = [r for r in results if (not episode or r.episode == episode)
                           and (not subtitleType or r.subtitleType == subtitleType)
                           and (not subtitleGroup or subtitleGroup in r.title)]
            data[searchHash] = {"results": results, "timestamp": timestamp}
            logging.info(
                f"Saved {query} to cache. Timestamp: {data[searchHash]['timestamp']}")

        data["results"] = results
        data.save()
        for i, r in enumerate(results):
            print(f"{i:<4} -> {r.title}")

    def download(self, index: int) -> None:
        '''
        Download file by index.
        '''
        result = data["results"][index]
        torrent_file = TorrentDownloader(result.url, ".")
        torrent_file.start_download()

        subscribed_animes = data.get("subscribed_animes", [])
        anime = [i for i in subscribed_animes if i in result.title]

        if len(anime) > 1:
            logging.warning(
                "More than one result found. Defaulting to the first one.")
        if len(anime) == 0:
            logging.debug("No results found.")
        else:
            anime = anime[0]
            data["subscribed_animes"][anime]["max_episode"] = max(
                data["subscribed_animes"][anime]["max_episode"], int(result.episode))

    def subscribe(self, query: str) -> int:
        '''
        Subscribe to an anime.
        '''
        subscribed_animes = data.get("subscribed_animes", {})
        if query in subscribed_animes:
            print("Already subscribed.")
            return 0
        else:
            subscribed_animes[query] = {}
            subscribed_animes[query]["max_episode"] = 0
            data["subscribed_animes"] = subscribed_animes
            data.save()
            print("You have successfully subscribed.")
            return 1

    def upgrade(self) -> int:
        '''
        Check for updates for each subscribed anime.
        '''
        subscribed_animes = data.get("subscribed_animes", [])
        if not subscribed_animes:
            print("No subscribed anime found.")
            return 0

        for anime in subscribed_animes:

            max_episode = data["subscribed_animes"][anime]["max_episode"]
            results = self.source.search(anime)
            available_max_episode = 0
            for r in results:
                if r.episode and r.episode.isdigit() and int(r.episode) > available_max_episode:
                    available_max_episode = int(r.episode)
            if available_max_episode > max_episode:
                print(f"{anime} : {max_episode} -> {available_max_episode}")
        return 1

    def show_subscribed(self) -> int:
        '''
        Show all subscribed anime and their max_episode.
        '''
        subscribed_animes = data.get("subscribed_animes", [])
        if not subscribed_animes:
            print("No subscribed anime found.")
            return 0

        for anime in subscribed_animes:
            max_episode = data["subscribed_animes"][anime]["max_episode"]
            print(f"{anime} -> {max_episode}")
        return 1


def main():

    bgmiget = BgmiGet(source=MiKanProject)
    fire.Fire(bgmiget)



import re
import logging
from functools import cached_property

logging.basicConfig(filename='bgmiget.log', level=logging.DEBUG)

class Result:
    '''
    Parse metadata to better display search results.
    '''
    def __init__(self, title: str, url: str) -> None:
        self.title = title
        self.url = url

    @cached_property
    def subtitleType(self) -> str:
        '''
        Parse subtitle language from title.
        '''
        return "tc" if ("繁" in self.title and "简" not in self.title) else "sc"

    @cached_property
    def episode(self) -> str:
        '''
        Parse episode from title.
        '''
        if "NCOPED" in self.title:
            return "NCOPED"
        if "OVA" in self.title:
            return "OVA"
        pattern = re.compile(r'(?![^ \[]])(第|未删减|_)?(?P<episode>\d+)(话|話|集|v2|先行版)?(?![^ \]])')
        match = pattern.search(self.title)
        try:
            matched = match.group("episode")
            logging.debug(f"Successfully parsed episode {matched} from {self.title}")
            return str(int(matched)) if matched.isdigit() else matched
        except:
            logging.warning(f"Failed to parse {self.title}. The value of match is {match}")
            return 0

    def __str__(self):
        return f"Title: {self.title}, Subtitle: {self.subtitleType}, Episode: {self.episode}, URL: {self.url}"

    def __repr__(self):
        return f"Result(title={self.title}, subtitle={self.subtitleType}, episode={self.episode}, url={self.url})"



import os
import pickle
from collections import UserDict

class PersistentDict(UserDict):
    def __init__(self, filename):
        self.filename = filename

        if os.path.exists(filename):
            with open(filename, "rb") as f:
                data = pickle.load(f)
                super().__init__(data)
        else:
            super().__init__()

    def save(self):
        with open(self.filename, "wb") as f:
            pickle.dump(self.data, f)


from typing import List

import requests
from bs4 import BeautifulSoup

from ..result import Result

BASE_URL = "https://mikanani.me/"


class MiKanProject:

    def search(self, query: str) -> List[Result]:

        URL = BASE_URL + "Home/Search?searchstr=" + query
        page = requests.get(URL)
        soup = BeautifulSoup(page.content, "html.parser")

        results = []
        for i in soup.find_all("tr", class_="js-search-results-row"):
            title = i.find("a", class_="magnet-link-wrap").text
            url = i.find(
                "a", class_="js-magnet magnet-link")["data-clipboard-text"]
            result = Result(title, url)
            results.append(result)
        return results
